; ***********************************
; ***  Borderless Bouncy Sprites  ***
; ***     with Chicklet Logo      ***
; ***     24-Sep-2024 by KPK      ***
; ***********************************

!addr spr_mem = $2000 ; target location for sprite data (4096)
!addr zp_ptr  = $fb   ; zero-page pointer
sprites_y     = $78   ; baseline y location for sprites
*             = $801  ; Start of Basic (2049)
!hex 20080a009e323038323a8f2032342d5345502d32303234204259204b504b000000

; --------------------------------------------------------------------

start:
  lda #$00    ; black
  sta $d020   ; border color
  sta $d021   ; background color
  lda #$93    ; clear screen
  jsr $ffd2

clear_sprite_data:
  ldy #$00
  tya
- sta spr_mem,y
  sta spr_mem+$100,y
  iny
  bne -

setup_sprite_0:
  ldy #$3a
- lda chicklet_sprite,y
  sta spr_mem,y
  dey
  bpl -

setup_sprites_1to7:
  lda $dc0e
  and #$fe
  sta $dc0e  ; turn off keyscan interrupt timer
  lda $01
  and #$fb
  sta $01    ; switch in character
  ldx #$07
  ldy #$18
- lda $d000+8*54,x   ; char pattern for '6'
  sta spr_mem+64,y
  lda $d000+8*52,x   ; char pattern for '4'
  sta spr_mem+64*2,y
  lda $d000+8*12,x   ; char pattern for 'L'
  sta spr_mem+64*3,y
  lda $d000+8*9,x    ; char pattern for 'I'
  sta spr_mem+64*4,y
  lda $d000+8*22,x   ; char pattern for 'V'
  sta spr_mem+64*5,y
  lda $d000+8*5,x    ; char pattern for 'E'
  sta spr_mem+64*6,y
  lda $d000+8*19,x   ; char pattern for 'S'
  sta spr_mem+64*7,y
  dey
  dey
  dey
  dex
  bpl -
  lda $01
  ora #$04
  sta $01    ; switch in io
  lda $dc0e
  ora #$01
  sta $dc0e  ; restart keyscan interrupt timer

  ; set sprite data pointers
  ldx #spr_mem/$40+$07
  ldy #$07
- txa
  sta $7f8,y  ; sprite n data pointer
  dex
  dey
  bpl -

  ; set sprite x locations
  ldx #$07
  ldy #$0e
- lda sprites_x,x
  sta $d000,y ; sprite n x pos
  dey
  dey
  dex
  bpl -

  ; set sprite colors
  ldy #$07
  lda #$01    ; white
- sta $d027,y ; sprite n color
  dey
  bpl -
  lda #$0b    ; medium gray
  sta $d027   ; adjust sprite 0 since it is multicolor
  lda #$0f    ; light gray
  sta $d026   ; sprites mc reg 1
  lda #$01
  sta $d01c   ; sprites mc mode
  sta $d025   ; sprites mc reg 0
  lda #%11111110 ; expand only sprites 1-7
  sta $d01d   ; sprites expand x
  sta $d017   ; sprites expand y
  lda #%11111111 ; enable all sprites
  sta $d015   ; sprites enable

; --------------------------------------------------------------------

loop1:        ; set border and background colors dynamically
  sei
  jsr animate_sprites
  lda #$5a    ; initial scan line (90)
  sta $fb
  ldx #$0f    ; prepare to display 16 colors
loop2:
- lda $d011   ; check for scan lines < 256
  bmi -
  lda $fb     ; wait for scan line 90
- cmp $d012
  bne -
  clc         ; one row down (8 scan lines)
  adc #$08
  sta $fb
  inc $d020   ; increment border color
  inc $d021   ; increment background color
  dex
  bpl loop2   ; repeat for all 16 colors
  bmi loop1   ; continue endlessly for each frame

; --------------------------------------------------------------------

animate_sprites:
  ldy #$07
-- ldx sine_offsets,y
  lda sine_table,x
  adc #sprites_y
  pha
  tya
  asl
  tax
  pla
  sta $d001,x
  tya
  tax
  dec sine_offsets,x
  bpl +
  lda #$1e
  sta sine_offsets,x
+ tya
  asl
  tax
  dec $d000,x
  lda $d000,x
  cmp #$ff
  bne +
  tya
  pha
  iny
  lda #$00
  sec
- rol
  dey
  bne -
  eor $d010
  sta $d010
  pla
  tay
+ dey
  bpl --
  rts

; --------------------------------------------------------------------

sprites_x: ; x locations for all eight sprites
!byte 98,132,148,180,196,212,228,244

chicklet_sprite:
!byte $00,$00,$00,$03,$fb,$00,$07,$7e
!byte $00,$35,$df,$00,$1d,$77,$00,$b7
!byte $5d,$00,$bd,$83,$7e,$ef,$01,$de
!byte $bb,$01,$78,$ae,$03,$70,$eb,$00
!byte $00,$ba,$03,$60,$ee,$03,$d8,$fb
!byte $02,$f6,$fe,$83,$bd,$9f,$ba,$00
!byte $37,$ee,$00,$3d,$fb,$00,$07,$7e
!byte $00,$03,$df

sine_offsets: ; each sprite is located in a different phase of the cycle
!byte 0,4,8,12,16,20,24,28

sine_table:
!byte 0,0,1,1,2,3,5,6,8,11,14,18,22,26,28,30
!byte 28,26,22,18,14,11,8,6,5,3,2,1,1,0,0