; ***********************************
; ***  Borderless Bouncy Sprites  ***
; ***     with Chicklet Logo      ***
; ***     28-Sep-2024 by KPK      ***
; ***********************************

!addr spr_mem = $2000 ; target location for sprite data (4096)
!addr zp_ptr  = $fb   ; zero-page pointer
sprites_y     = $74   ; baseline y location for sprites

* = $801 ; Start of Basic (2049)
!hex 20080a009e323038323a8f2032382d5345502d32303234204259204b504b000000

; ---------------------------------------------------------------------

start:
  lda #$00    ; black
  sta $d020   ; border color
  sta $d021   ; background color
  lda #$93    ; clear screen
  jsr $ffd2

paint_stripes: ; paint five lines of stripes on the screen (40 * 5 = 200 chars)
  ldy #56
  lda #160    ; inverted space character
- sta 1024+40*9-56,y ; nine lines down from the top of the screen
  iny
  bne -

paint_colors: ; paint colors over stripes
  lda #$68
  sta zp_ptr
  lda #$d9
  sta zp_ptr+1
  ldy #0
  ldx #5
-- lda raster_colors,x
- sta (zp_ptr),y
  iny
  cpy #40
  bne -
  lda zp_ptr
  clc
  adc #40
  sta zp_ptr
  bcc +
  inc zp_ptr+1
+ dex
  bpl --

clear_sprite_data:
  ldy #$00
  tya
- sta spr_mem,y
  sta spr_mem+$100,y
  iny
  bne -

setup_sprite_0:
  ldy #$3a
- lda chicklet_sprite,y
  sta spr_mem,y
  dey
  bpl -

setup_sprites_1to7:
  lda $dc0e
  and #$fe
  sta $dc0e  ; turn off keyscan interrupt timer
  lda $01
  and #$fb
  sta $01    ; switch in character
  ldx #$07
  ldy #$18
- lda $d000+8*54,x   ; char pattern for '6'
  sta spr_mem+64,y
  lda $d000+8*52,x   ; char pattern for '4'
  sta spr_mem+64*2,y
  lda $d000+8*12,x   ; char pattern for 'L'
  sta spr_mem+64*3,y
  lda $d000+8*9,x    ; char pattern for 'I'
  sta spr_mem+64*4,y
  lda $d000+8*22,x   ; char pattern for 'V'
  sta spr_mem+64*5,y
  lda $d000+8*5,x    ; char pattern for 'E'
  sta spr_mem+64*6,y
  lda $d000+8*19,x   ; char pattern for 'S'
  sta spr_mem+64*7,y
  dey
  dey
  dey
  dex
  bpl -
  lda $01
  ora #$04
  sta $01    ; switch in io
  lda $dc0e
  ora #$01
  sta $dc0e  ; restart keyscan interrupt timer

  ; set sprite data pointers
  ldx #spr_mem/$40+$07
  ldy #$07
- txa
  sta $7f8,y  ; sprite n data pointer
  dex
  dey
  bpl -

  ; set sprite x locations
  ldx #$07
  ldy #$0e
- lda sprites_x,x
  sta $d000,y ; sprite n x pos
  dey
  dey
  dex
  bpl -

  ; set sprite colors
  ldy #$07
  lda #$01    ; white
- sta $d027,y ; sprite n color
  dey
  bpl -
  lda #$0b    ; medium gray
  sta $d027   ; adjust sprite 0 since it is multicolor
  lda #$0f    ; light gray
  sta $d026   ; sprites mc reg 1
  lda #$01
  sta $d01c   ; sprites mc mode
  sta $d025   ; sprites mc reg 0
  lda #%11111110 ; expand only sprites 1-7
  sta $d01d   ; sprites expand x
  sta $d017   ; sprites expand y
  lda #%11111111 ; enable all sprites
  sta $d015   ; sprites enable

; ---------------------------------------------------------------------

loop1:        ; set border and background colors dynamically
  sei
  jsr animate_sprites
  lda #$7a    ; initial scan line
  sta zp_ptr
  ldx #$05    ; prepare to display 6 colors
loop2:
  lda zp_ptr  ; wait for target scan line
- cmp $d012
  bne -
  clc
  adc #$08    ; one row down (8 scan lines) for next color band
  sta zp_ptr
  lda raster_colors,x ; get raster color from table of values
  sta $d020   ; set border color
  dex
  bpl loop2   ; repeat for each color in table
  bmi loop1   ; continue endlessly for each frame

; ---------------------------------------------------------------------

animate_sprites:
  ldy #$07    ; prepare to adjust x/y positions of all eight sprites
-- ldx sine_offsets,y ; get offset into sine table for selected sprite
  lda sine_table,x ; get actual sine value (y position)
  adc #sprites_y ; add this to the y baseline for all sprites
  pha         ; save that value for now
  tya
  asl         ; there are two registers per sprite, so double the y value
  tax
  pla
  sta $d001,x ; save the previously-calculated y value for selected sprite
  tya         ; transfer y -> x (sprite number)
  tax
  dec sine_offsets,x ; decrement the offset for selected sprite
  bpl +
  lda #$1e    ; if the beginning has been reached, point to the end
  sta sine_offsets,x
+ tya
  asl         ; there are two registers per sprite, so double the y value
  tax
  dec $d000,x ; move the selected sprite one pixel left
  lda $d000,x
  cmp #$ff    ; is the x position < 0?
  bne +       ; no, so skip
  tya         ; yes, so...
  pha         ; ...save selected sprite number for now
  iny         ; convert sprite number to two's-compliment value
  lda #$00
  sec
- rol
  dey
  bne -
  eor $d010   ; toggle msb bit for affected sprite
  sta $d010
  pla         ; retrieve previously-saved sprite number
  tay
+ dey
  bpl --      ; repeat until all eight sprites have been moved
  rts

; ---------------------------------------------------------------------

sprites_x: ; x locations for all eight sprites
!byte 98,132,148,180,196,212,228,244

chicklet_sprite:
!byte $00,$00,$00,$03,$fb,$00,$07,$7e
!byte $00,$35,$df,$00,$1d,$77,$00,$b7
!byte $5d,$00,$bd,$83,$7e,$ef,$01,$de
!byte $bb,$01,$78,$ae,$03,$70,$eb,$00
!byte $00,$ba,$03,$60,$ee,$03,$d8,$fb
!byte $02,$f6,$fe,$83,$bd,$9f,$ba,$00
!byte $37,$ee,$00,$3d,$fb,$00,$07,$7e
!byte $00,$03,$df

sine_offsets: ; each sprite is located in a different phase of the cycle
!byte 0,4,8,12,16,20,24,28

sine_table: ; y values to be added to the baseline for selected sprite
!byte 0,0,1,1,2,3,5,6,8,11,14,18,22,26,28,30
!byte 28,26,22,18,14,11,8,6,5,3,2,1,1,0,0

raster_colors: ; commodore colors (in reverse order)
!byte $00,$06,$05,$07,$08,$02