; ***********************************
; ***  Borderless Bouncy Sprites  ***
; ***   with Chicklet Logo ISR    ***
; ***      1-Oct-2024 by KPK      ***
; ***********************************

!addr spr_mem   = $2000 ; target location for sprite data (8192)
!addr band_no   = $fb   ; number of color band (horizontal stripe)
!addr sprite_no = $fc   ; currently-selected sprite for animation
sprites_y       = $7f   ; baseline y location for sprites

* = $801 ; Start of Basic (2049)
!hex 1f080a009e323038313a8f20312d4f43542d32303234204259204b504b000000

; --------------------------------------------------------------------

start:
  lda #$00    ; black
  sta $d020   ; border color
  sta $d021   ; background color
  lda #$01    ; white
  sta $286    ; cursor color
  lda #$93    ; clear screen
  jsr $ffd2

; clear target memory for sprites before copying from rom
clear_sprite_data:
  ldy #$00
  tya
- sta spr_mem,y
  sta spr_mem+$100,y
  iny
  bne -

; copy sprite 0 (chicklet logo) to target memory location
setup_sprite_0:
  ldy #$3a
- lda chicklet_sprite,y
  sta spr_mem,y
  dey
  bpl -

; copy sprites 1-7 from character rom
setup_sprites_1to7:
  lda $dc0e
  and #$fe
  sta $dc0e   ; turn off keyscan interrupt timer
  lda $01
  and #$fb
  sta $01     ; switch in character
  ldx #$07
  ldy #$18
- lda $d000+8*54,x   ; char pattern for '6'
  sta spr_mem+64,y
  lda $d000+8*52,x   ; char pattern for '4'
  sta spr_mem+64*2,y
  lda $d000+8*12,x   ; char pattern for 'L'
  sta spr_mem+64*3,y
  lda $d000+8*9,x    ; char pattern for 'I'
  sta spr_mem+64*4,y
  lda $d000+8*22,x   ; char pattern for 'V'
  sta spr_mem+64*5,y
  lda $d000+8*5,x    ; char pattern for 'E'
  sta spr_mem+64*6,y
  lda $d000+8*19,x   ; char pattern for 'S'
  sta spr_mem+64*7,y
  dey
  dey
  dey
  dex
  bpl -
  lda $01
  ora #$04
  sta $01     ; switch in io
  lda $dc0e
  ora #$01
  sta $dc0e   ; restart keyscan interrupt timer

; set sprite data pointers
  ldx #spr_mem/$40+$07
  ldy #$07
- txa
  sta $7f8,y  ; sprite n data pointer
  dex
  dey
  bpl -

; set sprite x locations
  ldx #$07
  ldy #$0e
- lda sprites_x,x
  sta $d000,y ; sprite n x pos
  dey
  dey
  dex
  bpl -

; set sprite colors
  ldy #$07
  lda #$01    ; white
- sta $d027,y ; sprite n color
  dey
  bpl -
  lda #$0b    ; medium gray
  sta $d027   ; adjust sprite 0 since it is multicolor
  lda #$0f    ; light gray
  sta $d026   ; sprites mc reg 1
  lda #$01
  sta $d01c   ; sprites mc mode
  sta $d025   ; sprites mc reg 0
  lda #%11111110 ; expand only sprites 1-7
  sta $d01d   ; sprites expand x
  sta $d017   ; sprites expand y
  lda #%11111111 ; enable all sprites
  sta $d015   ; sprites enable

; --------------------------------------------------------------------

setup_isr:    ; set up interrupt service routine
  sei         ; disable interrupt
  lda #<isr   ; set interrupt routine to c01f
  sta $0314   ; set low byte
  lda #>isr   ; high byte
  sta $0315   ; set
  lda $d011   ; and for the high bit of the raster position
  and #$7f    ; clear the high bit
  sta $d011   ; and set the cleared high bit
  lda #$f1    ; enable the raster interrupt
  sta $d01a   ; in the appropriate register
  cli         ; allow interrupt
  rts         ; return from subroutine

; --------------------------------------------------------------------

isr:
  lda $d019   ; load vic interrupt register
  sta $d019   ; and clear it
  bmi +       ; if highest bit is set, go to our routine
  lda $dc0d   ; else disable cia interrupt
  cli         ; enable interrupt
  jmp $ea31   ; continue to normal system isr vector

+ jsr animate_sprite  ; move one sprite one pixel to the left
  ldx band_no         ; get the color band number
  lda raster_values,x ; get the raster number for the color band
  cmp $d012   ; compare it to the vic-ii raster number
  bcc +       ; skip if it's less
  lda raster_colors,x ; otherwise get the associated color value
  sta $d020   ; set the border color
  sta $d021   ; set the background color
+ dec band_no ; select the next band
  bpl +       ; if the beginning has been reached...
  lda #$06    ; ...point to the end
  sta band_no
+ lda raster_values,x ; get the next raster number
  sta $d012   ; tell vic-ii chip to generate an interrupt when reached
  jmp $ea81   ; continue to system isr vector

; --------------------------------------------------------------------

animate_sprite: ; move selected sprite one pixel to the left
  inc sprite_no ; select next sprite to be moved
  lda sprite_no
  and #$07
  sta sprite_no
  tay         ; prepare to adjust x/y positions of selected sprite
  ldx sine_offsets,y ; get offset into sine table for selected sprite
  lda sine_table,x ; get actual sine value (y position)
  adc #sprites_y ; add this to the y baseline for selected sprite
  pha         ; save that value for now
  tya
  asl         ; there are two registers per sprite, so double the y value
  tax
  pla
  sta $d001,x ; save the previously-calculated y value for selected sprite
  tya         ; transfer y -> x (sprite number)
  tax
  dec sine_offsets,x ; decrement the offset for selected sprite
  bpl +
  lda #$1e    ; if the beginning has been reached, point to the end
  sta sine_offsets,x
+ tya
  asl         ; there are two registers per sprite, so double the y value
  tax
  dec $d000,x ; move the selected sprite one pixel left
  lda $d000,x
  cmp #$ff    ; is the x position < 0?
  bne +       ; no, so skip
  lda #$00    ; yes, so convert sprite number to msb bit number
  sec
- rol
  dey
  bpl -
  eor $d010   ; toggle msb bit for selected sprite
  sta $d010
+ rts

; --------------------------------------------------------------------

sprites_x:    ; x locations for sprites
!byte 98,132,148,180,196,212,228,244

sine_offsets: ; phase of cycle for each sprite
!byte 0,4,8,12,16,20,24,28

sine_table:
!byte 0,0,1,1,2,3,5,6,8,11,14,18,22,26,28,30
!byte 28,26,22,18,14,11,8,6,5,3,2,1,1,0,0

raster_colors: !byte $00,$06,$05,$07,$08,$02
raster_values: !byte $b2,$aa,$a2,$9a,$92,$8a,$82

chicklet_sprite:
!byte $00,$00,$00,$03,$fb,$00,$07,$7e
!byte $00,$35,$df,$00,$1d,$77,$00,$b7
!byte $5d,$00,$bd,$83,$7e,$ef,$01,$de
!byte $bb,$01,$78,$ae,$03,$70,$eb,$00
!byte $00,$ba,$03,$60,$ee,$03,$d8,$fb
!byte $02,$f6,$fe,$83,$bd,$9f,$ba,$00
!byte $37,$ee,$00,$3d,$fb,$00,$07,$7e
!byte $00,$03,$df