; Bouncy Sprites with Chicklet Logo
; September 22, 2024 by KPK

* = $801

!addr spr_mem = $2000 ; target location for sprite data (4096)
!addr zp_ptr  = $fb   ; zero-page pointer

sprites_y = 120       ; baseline y location for sprites

!hex 20080a009e323038323a8f2032322d5345502d32303234204259204b504b000000

; --------------------------------------------------------------------

start:
  lda #0      ; set background color to black
  sta $d021
  lda #11     ; set border color to medium gray
  sta $d020
  lda #1      ; set cursor color to white
  sta $286
  lda #147    ; clear screen
  jsr $ffd2

paint_stripes: ; paint stripes on screen
  ldy #0
  lda #160    ; inverted space character
- sta 1024+200,y
  sta 1024+456,y
  sta 1024+544,y
  iny
  bne -

paint_colors: ; paint colors over stripes
  lda #$c8
  sta zp_ptr
  lda #$d8
  sta zp_ptr+1
  ldx #1      ; color to paint
  ldy #0
pc_loop:
  txa
- sta (zp_ptr),y
  iny
  cpy #40
  bne -
  lda zp_ptr
  clc
  adc #40
  sta zp_ptr
  bcc +
  inc zp_ptr+1
+ inx
  cpx #16
  bne pc_loop

clear_sprite_data:
  ; clear sprite data in preparation for copying the left-most column...
  ; ...of bytes from character rom (for sprites 1-7)
  lda #<spr_mem
  sta zp_ptr
  lda #>spr_mem
  sta zp_ptr+1
  ldx #2
  ldy #0
  tya
- sta (zp_ptr),y
  iny
  bne -
  inc zp_ptr+1
  dex
  bne -

setup_sprite_0:
  ldy #0
- lda chicklet_sprite,y
  sta spr_mem,y
  iny
  cpy #64
  bne -

setup_sprites_1to7:
  lda $dc0e
  and #$fe
  sta $dc0e  ; turn off keyscan interrupt timer
  lda $01
  and #$fb
  sta $01    ; switch in character
  ldx #0
  ldy #0
- lda $d000+8*54,x   ; char pattern for '6'
  sta spr_mem+64,y
  lda $d000+8*52,x   ; char pattern for '4'
  sta spr_mem+64*2,y
  lda $d000+8*12,x   ; char pattern for 'L'
  sta spr_mem+64*3,y
  lda $d000+8*9,x    ; char pattern for 'I'
  sta spr_mem+64*4,y
  lda $d000+8*22,x   ; char pattern for 'V'
  sta spr_mem+64*5,y
  lda $d000+8*5,x    ; char pattern for 'E'
  sta spr_mem+64*6,y
  lda $d000+8*19,x   ; char pattern for 'S'
  sta spr_mem+64*7,y
  iny
  iny
  iny
  inx
  cpx #8
  bne -
  lda $01
  ora #$04
  sta $01    ; switch in io
  lda $dc0e
  ora #$01
  sta $dc0e  ; restart keyscan interrupt timer

  ; set sprite data pointers
  ldx #spr_mem / 64
  ldy #0
- txa
  sta $7f8,y  ; sprite n data pointer
  inx
  iny
  cpy #8
  bne -

  ; set sprite x locations
  ldx #0
  ldy #0
- lda sprites_x,x
  sta $d000,y ; sprite n x pos
  inx
  iny
  iny
  cpy #16
  bne -

  ; set sprite colors
  ldy #0
  lda #1      ; white
- sta $d027,y ; sprite n color
  iny
  cpy #8
  bne -
  lda #11     ; medium gray
  sta $d027   ; adjust sprite 0 since it is multicolor
  lda #15     ; light gray
  sta $d026   ; sprites mc reg 1
  lda #1
  sta $d01c   ; sprites mc mode
  sta $d025   ; sprites mc reg 0
  lda #%11111110 ; expand only sprites 1-7
  sta $d01d   ; sprites expand x
  sta $d017   ; sprites expand y
  lda #%11111111 ; enable all sprites
  sta $d015   ; sprites enable

; --------------------------------------------------------------------

main_loop:
  jsr animate_sprites
  ldx #0      ; crude time delay
- nop
  dex
  bne -
  jsr $ffe4   ; check for keypress
  beq main_loop
  lda #0
  sta $d015   ; turn off all sprites
  lda #147    ; clear screen
  jsr $ffd2
  rts         ; exit program

animate_sprites:
  ldy #0
- ldx sine_offsets,y
  lda sine_table,x
  adc #sprites_y
  pha
  tya
  asl
  tax
  pla
  sta $d001,x
  tya
  tax
  dec sine_offsets,x
  iny
  cpy #8
  bne -
  rts

; --------------------------------------------------------------------

sprites_x: ; x locations for all eight sprites
!byte 98,132,148,180,196,212,228,244

chicklet_sprite:
!byte $00,$00,$00,$03,$fb,$00,$07,$7e
!byte $00,$35,$df,$00,$1d,$77,$00,$b7
!byte $5d,$00,$bd,$83,$7e,$ef,$01,$de
!byte $bb,$01,$78,$ae,$03,$70,$eb,$00
!byte $00,$ba,$03,$60,$ee,$03,$d8,$fb
!byte $02,$f6,$fe,$83,$bd,$9f,$ba,$00
!byte $37,$ee,$00,$3d,$fb,$00,$07,$7e
!byte $00,$03,$df,$00,$00,$00,$00,$00

sine_offsets: ; relative y offsets for all eight sprites
!byte $00,$30,$50,$70,$90,$b0,$d0,$f0

sine_table:
!hex 111111121212131314141415151616161717171818181919191a1a1a1b1b1b1c
!hex 1c1c1c1d1d1d1d1e1e1e1e1e1f1f1f1f1f1f2020202020202020202020202020
!hex 2020202020202020202020202020201f1f1f1f1f1f1e1e1e1e1e1d1d1d1d1c1c
!hex 1c1b1b1b1b1a1a1a191919181818171716161615151514141313131212111111
!hex 1010100f0f0e0e0e0d0d0c0c0c0b0b0b0a0a0909090808080707070606060605
!hex 0505040404040303030303020202020202010101010101010101010101010101
!hex 0101010101010101010101010101020202020202030303030304040404050505
!hex 050606060707070808080909090a0a0a0b0b0b0c0c0d0d0d0e0e0f0f0f101010